unit fdbtPluginMain;

interface

uses Windows, SysUtils, Classes, TagEditorLibConsts, TagEditorLibAddinConsts,
  fdbtPluginUtils, fdbtPlugin, KLSFile,ID3v2;



var
  // W
  GenreListData: TTagListData;

const
  FDBTP_INFO: array[0..FDBT_AUTHOR] of string = (
        '0.9',
        'KLS',
        '.kls',
        '34430'
        );


  FDBT_FIELD_LENGTH      = 'length';
  FDBT_FIELD_BITRATE     = 'bitrate';
  FDBT_FIELD_SAMPLERATE  = 'SampleRate';
  FDBT_FIELD_CHANNELS    = 'channels';

  FDBT_FIELD_RATIO       = 'Ratio';


  FDBTP_FIELDCOUNT = 13;
  FDBTP_FIELDS: array[0..FDBTP_FIELDCOUNT-1] of TTagField = (
        (
        Name:        FDBT_FIELD_TITLE;
        DisplayName: '^Cg';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_ARTIST;
        DisplayName: 'A[eBXg';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_ALBUM;
        DisplayName: 'Ao';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_TRACK;
        DisplayName: 'gbN';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_DURATION;
        DisplayName: 't';
        ValueType:   FDBT_VALUE_TYPE_DOUBLE;
        Editable:    False;      Visible: False;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_GENRE;
        DisplayName: 'W';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_YEAR;
        DisplayName: '';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_COMMENT;
        DisplayName: 'Rg';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    True;      Visible: True;
        Data:        nil;
        ),

        (
        Name:        FDBT_FIELD_LENGTH;
        DisplayName: '';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    False;     Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_SAMPLERATE;
        DisplayName: 'Tv[g';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    False;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_CHANNELS;
        DisplayName: '`l';
        ValueType:   FDBT_VALUE_TYPE_INTEGER;
        Editable:    False;      Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_BITRATE;
        DisplayName: 'rbg[g';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    False;     Visible: True;
        Data:        nil;
        ),
        (
        Name:        FDBT_FIELD_RATIO;
        DisplayName: 'k';
        ValueType:   FDBT_VALUE_TYPE_STRING;
        Editable:    False;      Visible: True;
        Data:        nil;
        )
        );


type

TFdbtPluginMain = class(TFdbtPlugin)
private
  FAddinMenu: TAddinMenu;
  FGetTrackDataInfo: TGetTrackDataInfo;
public
  TrackTools: TTrackDataInfo;
  function Initialize(info: PFdbtPluginInitInfo): integer; override;
  function GetLibInfo(InfoNo: Integer; Buffer: PChar; Size: Integer): Integer; override;
  function ShowConfig: integer; override;
  function GetTagInfo(FileName: PWideChar; Tag: PTagInfo; Size: integer): integer; override;
  function SetTagInfo(FileName: PWideChar; Tag: PTagInfo): integer; override;
  function GetTagFieldInfo(fl: PTagFieldList; size: integer): integer; override;

  function AddinMenu(place: integer; Identify: PChar; DisplayName: PChar; ProcExe: TAddinProc; ImageType: integer; Image: cardinal): integer;
  function GetTrackDataInfo: TTrackDataInfo;
end;


implementation



function TFdbtPluginMain.Initialize(info: PFdbtPluginInitInfo): integer;
begin
  inherited Initialize(info);
  @FAddinMenu:=InitInfo.GetService('AddinMenu');
  @FGetTrackDataInfo:=InitInfo.GetService('GetTrackDataInfo');

  TrackTools:=GetTrackDataInfo();

  Result:=FDBT_SUCCESS;
end;


function TFdbtPluginMain.GetLibInfo(InfoNo: Integer; Buffer: PChar; Size: Integer): Integer;
var
  PlugInfo : string;
begin
  PlugInfo:=FDBTP_INFO[InfoNo];
  lstrcpyn(Buffer, PChar(PlugInfo), Size);
  Result:=Length(Buffer);
end;

function TFdbtPluginMain.ShowConfig: integer;
begin
  Result:=FDBT_NOTIMPLEMENTED;
end;

function TFdbtPluginMain.GetTagInfo(FileName: PWideChar; Tag: PTagInfo; Size: integer): integer;
var
  tg: TKLS;
  v2: TID3v2;
begin
  tg:=TKLS.Create;

  if tg.ReadFromFile(FileName) then
  begin
    v2:=tg.ID3v2;
    if v2.Exists then
    begin
      SetTagValue(tag^[0], FDBT_FIELD_TITLE,    v2.Title);
      SetTagValue(tag^[1], FDBT_FIELD_ARTIST,   v2.Artist);
      SetTagValue(tag^[2], FDBT_FIELD_ALBUM,    v2.Album);
      SetTagValue(tag^[3], FDBT_FIELD_TRACK,    v2.TrackString);
      SetTagValue(tag^[4], FDBT_FIELD_GENRE,    v2.Genre);
      SetTagValue(tag^[5], FDBT_FIELD_YEAR,     v2.Year);
      SetTagValue(tag^[6], FDBT_FIELD_COMMENT,  v2.Comment);
    end;
    //܂łID3v2^O̓ǂݍ

    SetTagValue(Tag^[7], FDBT_FIELD_DURATION, tg.Duration);

    SetTagValue(Tag^[8],  FDBT_FIELD_CHANNELS,    tg.Channels);
    SetTagValue(Tag^[9],  FDBT_FIELD_LENGTH,      DurationToStr(Round(tg.Duration), False));
    SetTagValue(Tag^[10], FDBT_FIELD_BITRATE,     Format('%f kbit/s', [(tg.Bitrate/1000)]));
    SetTagValue(Tag^[11], FDBT_FIELD_SAMPLERATE,  Format('%d hz', [tg.SampleRate]));
    SetTagValue(Tag^[12], FDBT_FIELD_RATIO,       Format('%f %%', [tg.Ratio]));

    Result:=14;
  end else
    Result:=0;

  tg.Free;
end;

function TFdbtPluginMain.SetTagInfo(FileName: PWideChar; Tag: PTagInfo): integer;
var
  tg: TKLS;
  i,c: integer;
  v2: TID3v2;
  fld, v: string;
begin

  tg:=TKLS.Create;
  v2:=tg.ID3v2;
  c:=Length(Tag^);

  for i:=0 to c-1 do
  begin
    fld := Tag^[i].Field;
    v   := Value2String(Tag^[i].Value);

    if fld=FDBT_FIELD_TITLE   then v2.Title   := v;
    if fld=FDBT_FIELD_ARTIST  then v2.Artist  := v;
    if fld=FDBT_FIELD_ALBUM   then v2.Album   := v;
    if fld=FDBT_FIELD_TRACK   then v2.Track   := StrToIntDef(v, 0);
    if fld=FDBT_FIELD_GENRE   then v2.Genre   := v;
    if fld=FDBT_FIELD_YEAR    then v2.Year    := v;
    if fld=FDBT_FIELD_COMMENT then v2.Comment := v;

  end;

  // 
  if v2.SaveToFile(FileName) then
  begin
    Result:=FDBT_SUCCESS;
  end else
  begin
    Result:=FDBT_INTERNAL_ERROR;
  end;

  tg.Free;
end;

function TFdbtPluginMain.GetTagFieldInfo(fl: PTagFieldList; size: integer): integer;
var
  i: integer;
begin
  for i:=0 to FDBTP_FIELDCOUNT-1 do
    fl^[i]:=FDBTP_FIELDS[i];

  Result:=FDBTP_FIELDCOUNT;
end;

function TFdbtPluginMain.AddinMenu(place: integer; Identify: PChar; DisplayName: PChar; ProcExe: TAddinProc; ImageType: integer; Image: cardinal): integer;
begin
  if Assigned(FAddinMenu) then
    Result:=FAddinMenu(place, Identify, DisplayName, ProcExe, ImageType, Image)
  else
    Result:=FDBT_NOTIMPLEMENTED;
end;

function TFdbtPluginMain.GetTrackDataInfo: TTrackDataInfo;
begin
  if Assigned(FGetTrackDataInfo) then Result:=FGetTrackDataInfo();
end;



initialization


end.
